package im.angry.openeuicc.vendored

import android.util.Log
import net.typeblog.lpac_jni.ApduInterface

data class ESTKmeInfo(
    val serialNumber: String?,
    val bootloaderVersion: String?,
    val firmwareVersion: String?,
    val skuName: String?,
)

fun getESTKmeInfo(iface: ApduInterface): ESTKmeInfo? {
    // @formatter:off
    val aid = byteArrayOf(
        0xA0.toByte(), 0x65.toByte(), 0x73.toByte(), 0x74.toByte(), 0x6B.toByte(), 0x6D.toByte(),
        0x65.toByte(), 0xFF.toByte(), 0xFF.toByte(), 0xFF.toByte(), 0xFF.toByte(), 0xFF.toByte(),
        0xFF.toByte(), 0x6D.toByte(), 0x67.toByte(), 0x74.toByte(),
    )
    // @formatter:on
    return try {
        iface.connect()
        iface.logicalChannelOpen(aid)
        fun transmit(p1: Byte) =
            decode(iface.transmit(byteArrayOf(0x00, 0x00, p1, 0x00, 0x00)).clone())
        return ESTKmeInfo(
            transmit(0x00), // serial number
            transmit(0x01), // bootloader version
            transmit(0x02), // firmware version
            transmit(0x03), // sku name
        )
    } catch (e: Exception) {
        Log.d("ESTKme", "Failed to get ESTKme info", e)
        null
    }
}

private fun decode(b: ByteArray): String? {
    if (b.size < 2) return null
    if (b[b.size - 2] != 0x90.toByte() || b[b.size - 1] != 0x00.toByte()) return null
    return b.sliceArray(0 until b.size - 2).decodeToString()
}
